'use strict'
// es6

function check(fn, arg, expected) {
    const actual = fn(arg)
    if (actual !== expected) {
        throw new Error('Expected [' + expected + '] actual [' + actual + '] on input [' + arg + ']')
    }
}

function domain(raw) {
    // Extracts something that looks like a domain from given string.
    //
    // Setting the href property on an anchor, then extracting hostname appears to do the
    // case-folding and normalization necessary for international and uppercase domains, though
    // I'm not sure where that's defined in spec.
    const a = document.createElement('a')
    // Only bare domains and http(s) urls considered. Everything else is invalid for this purpose.
    const normed = (raw || '')
        .replace(/^\s+/, '')
        // Firefox is pretty liberal about the number of slashes allowed, but behavior differs
        // depending on whether you're in a web page proper or a background script. It seems that in
        // a background script, hostname for http:example.com naturally resolves to 'example.com'
        // but in a page, say http://julfers.com, it gets treated as a relative url and resolves to
        // 'julfers.com'. Don't know if this is according to spec or consistent across browsers.
        // 
        // In any case, normalize slashes, so background/content script behavior is consistent. This
        // would be incorrect for file:///, but I don't care about that.
        .replace(/^([a-zA-Z]*:\/*)/, '$1//')
        .replace(/^(?![a-zA-Z]*:)/, 'http://')
    a.href = normed
    // When something isn't a valid url, the hostname attribute returns empty string.
    return /^https?:\/\//i.test(normed) && a.hostname !== ''
        ? a.hostname
        : null
}
// When these checks fail, it seems that if you're using the "load temporary addon" feature, you
// can't see the error in an existing debug window. That is, if you need to click "debug" _after_
// clicking "reload" to see the error output. Seems like a bug, but I haven't searched for it.
//
// Seems fixed on or around Firefox 57
check(domain, 'example.com', 'example.com')
check(domain, '  example.com  ', 'example.com')
check(domain, '  http://example.com  ', 'example.com')
check(domain, 'http:example.com', 'example.com')
check(domain, 'http:/example.com', 'example.com')
check(domain, 'http:////example.com', 'example.com')
check(domain, 'example.com/foo', 'example.com')
check(domain, 'example.com:80', 'example.com')
check(domain, 'example.com:80/foo', 'example.com')
check(domain, 'EXAMPLE.COM', 'example.com')
check(domain, 'http://example.com', 'example.com')
check(domain, 'https://example.com', 'example.com')
check(domain, 'HTTP://EXAMPLE.COM', 'example.com')
check(domain, 'http://localhost', 'localhost')
// nfkc Unicode normalization, per rfc3491:
// seems to have changed (broke?) in Firefox 57
// check(domain, 'http://\u1e9b\u0323\u0307.example.com', '\u1e69.example.com')
check(domain, 'ftp://example.com', null)
check(domain, 'http://  example.com', null)
check(domain, ':example.com', null)
check(domain, '::example.com', null)
check(domain, 'xhttp://example.com', null)

const settings = {whitelist: []}
browser.storage.local.get().then(
    s => { settings.whitelist = s.whitelist || [] },
    error => { throw error })

function onSettingsChange(fn) {
    browser.storage.onChanged.addListener(function (changes) {
        for (const k in changes) {
            settings[k] = changes[k].newValue || []
        }
        fn()
    })
}
onSettingsChange(() => {}) // no-op listener ensures that settings variable gets updated
